import 'package:blocstar/BlocBase.dart';

import 'CounterContext.dart';

class CounterBloc extends BlocBase<CounterContext> {
  @override
  Future initializeAsync() async {
    context = new CounterContext(
        count: 0,
        description: "Button Press Count",
        onContextChanged: onContextChangedCallback);
  }

  buttonPressedAsync(
    int duration,
  ) async {
    final incrementedCount = await runAsync(
        function: () async => await _incrementAsync(duration),
        timeoutSeconds: 3);

    //Timed out calls or calls in error, return null
    if (incrementedCount != null) {
      context = context.merge(newCount: incrementedCount);
    }
  }

  Future<int> _incrementAsync(int duration) async {
    final incrementedCount =
        await Future.delayed(new Duration(seconds: duration), () {
      return context.count + 1;
    });
    return incrementedCount;
  }

  simulateExceptionAsync() async {
    await runAsync(
        function: () async {
          await Future.delayed(
              new Duration(seconds: 2), () => throw new Exception("Bad Luck"));
        },
        timeoutSeconds: 3);
  }
}
